@@ -19,6 +19,9 @@ import torch.nn as nn
 import torch.nn.functional as F
 from torchvision import models
 from copy import deepcopy
+import mlflow
+
+
 from classification_analysis.classification_analysis import ClassificationAnalysis
 
 class ModelInitializer:
@@ -379,7 +382,7 @@ class ModelTraining:
                 
             self.update_phase_data()
     
-    def one_epoch(self, dataset, epoch, best):
+    def one_epoch(self, dataset, epoch, best, track):
         
         self.initialize_epoch_data()
         self.scheduler_step(best)
@@ -396,7 +399,7 @@ class ModelTraining:
         print('Current learning rate: {} '.format(self.optimizer.param_groups[0]['lr']))
         self.evaluation_metrics_calculation(dataset)
         self.evaluation_metrics_visualization()
-        self.model_save(best)
+        self.model_save(best, track)
 
     def evaluation_metrics_calculation(self, dataset):
         # Calculate the loss
@@ -445,7 +448,7 @@ class ModelTraining:
         ClassificationAnalysis.line_plot(self.metrics['train_spec'],      self.metrics['val_spec'],      'Specificity')
       
     
-    def model_save(self, best):
+    def model_save(self, best, track):
         def save_main_parameters(self):
             to_be_saved = dict()
             to_be_saved['epoch_data'] = self.epoch_data
@@ -491,6 +494,16 @@ class ModelTraining:
         file_name = self.model_name + '.pkl'
         should_be_saved = False
         if model_improved:
+            # Tracking Experiments 
+            if track:
+                self.to_be_logged = {
+                    'params': [('epoch', self.current_epoch), 
+                               ('loss_function', self.loss_function),
+                               ('scheduler', self.scheduler)],
+                               
+                    'metrics': [(key, self.metrics[key][-1]) for key in self.metrics]
+                }
+            
             should_be_saved = True
             to_save = {
                            'last_model': self.model if self.save_last_model else None,
@@ -520,7 +533,7 @@ class ModelTraining:
             with open(file_name, 'wb') as f:
                 pk.dump(to_save, f)
         
-    def loop(self, dataset, best = 'val_acc', reset_epoch_cnt=True):
+    def loop(self, dataset, best='val_acc', reset_epoch_cnt=True, track=True):
         
         if reset_epoch_cnt:
             # If we want continue training
@@ -528,9 +541,16 @@ class ModelTraining:
         
         for epoch in range(self.num_epochs):
             with ElapsedTime('One epoch').cpu(with_gpu=False):
-                self.one_epoch(dataset, epoch, best)
+                self.one_epoch(dataset, epoch, best, track)
                 
-    
+        if track:
+            with mlflow.start_run(run_name=self.model_name) as run:
+                for param_name, param_value in self.to_be_logged['params']:
+                    mlflow.log_param(param_name, param_value)
+                    
+                for metric_name, metric_value in self.to_be_logged['metrics']:
+                    mlflow.log_metric(metric_name, metric_value)
+                    
     def features_extraction_details(self, n_crops, source_layers, phase, iteration):
         clear_output()
         print('Number of crops is {}'.format(n_crops))
